//
//  MyView.m
//  QC3D
//
//  Created by Pierre Chatelier on 30/05/08.
//  Copyright 2008 __MyCompanyName__. All rights reserved.
//

#import "MyView.h"

#import "MyQCView.h"

#import <Quartz/Quartz.h>

@implementation MyView

-(id) initWithCoder:(NSCoder*)coder
{
  if (![super initWithCoder:coder])
    return nil;
  [self registerForDraggedTypes:[NSArray arrayWithObjects:NSFilenamesPboardType, nil]];
  return self;
}
//end initWithCoder:

-(NSDragOperation)draggingEntered:(id<NSDraggingInfo>)sender
{
  NSPasteboard* pasteboard = [sender draggingPasteboard];
  NSArray* fileNames = [pasteboard propertyListForType:NSFilenamesPboardType];
  NSString* fileName = [fileNames lastObject];
  return ([[fileName pathExtension] isEqualToString:@"qtz"] || [[fileName pathExtension] isEqualToString:@"obj"]) ?
         NSDragOperationCopy : NSDragOperationNone;
}
//end draggingEntered:

- (BOOL)performDragOperation:(id < NSDraggingInfo >)sender
{
  NSPasteboard* pasteboard = [sender draggingPasteboard];
  NSArray* fileNames = [pasteboard propertyListForType:NSFilenamesPboardType];
  NSString* fileName = [fileNames lastObject];
  NSPoint draggingLocation = [sender draggingLocation];
  draggingLocation = [self convertPoint:draggingLocation fromView:nil];
  MyQCView* qcView = [[MyQCView alloc] initWithFrame:NSMakeRect(draggingLocation.x-50, draggingLocation.y-50, 100, 100)];
  BOOL ok = NO;
  if ([[fileName pathExtension] isEqualToString:@"qtz"])
    ok = [qcView loadCompositionFromFile:fileName];
  else if ([[fileName pathExtension] isEqualToString:@"obj"])
  {
    ok = [qcView loadCompositionFromFile:[[NSBundle mainBundle] pathForResource:@"QC3DPlugin" ofType:@"qtz"]];
    if (ok)
      [qcView setValue:fileName forInputKey:@"inputModelFilepath"];
  }
  if (ok)
  {
    [self addSubview:qcView];
    [qcView release];
    [qcView startRendering];
  }
  else
  {
    [qcView release];
    qcView = nil;
  }
  return qcView ? YES : NO;
}
//end performDragOperation:

-(void) mouseDown:(NSEvent*)event
{
  NSPoint point = [self convertPoint:[event locationInWindow] fromView:nil];
  NSEnumerator* enumerator = [[self subviews] objectEnumerator];
  NSView* subView = nil;
  draggedView = nil;
  while(!draggedView && ((subView = [enumerator nextObject])))
      draggedView = ([subView isKindOfClass:[QCView class]] && NSPointInRect(point, [subView frame])) ? subView : nil;
}
//end mouseDown:

-(void) mouseUp:(NSEvent*)event
{
  draggedView = nil;
}
//end mouseUp:

-(void) mouseDragged:(NSEvent*)event
{
  if (draggedView)
  {
    if ([event modifierFlags] & NSCommandKeyMask)
    {
      QCView* qcView = [draggedView isKindOfClass:[QCView class]] ? (QCView*)draggedView : nil;
      [qcView setValue:[NSNumber numberWithFloat:[event deltaX]] forInputKey:@"inputDeltaX"];
      [qcView setValue:[NSNumber numberWithFloat:[event deltaY]] forInputKey:@"inputDeltaY"];
    }
    else
    {
      NSPoint newOrigin = [draggedView frame].origin;
      QCView* qcView = [draggedView isKindOfClass:[QCView class]] ? (QCView*)draggedView : nil;
      [qcView setValue:[NSNumber numberWithFloat:0] forInputKey:@"inputDeltaX"];
      [qcView setValue:[NSNumber numberWithFloat:0] forInputKey:@"inputDeltaY"];
      newOrigin.x += [event deltaX];
      newOrigin.y -= [event deltaY];
      [draggedView setFrameOrigin:newOrigin];
    }
  }
}
//end mouseDragged:

-(void)drawRect:(NSRect)rect
{
  [super drawRect:rect];
}
//end drawRect:

@end
